local _,stash = ...
local L = stash.localization
local C = stash.constants
local settings = stash.settings
stash.itemsPanel = stash.frame.itemsPanel

local displayList = {} -- ordered list of itemIDs to display

-- one-time setup called during initialization
function stash.itemsPanel:Setup()

    local scrollFrame = self.List
	scrollFrame.template = "StashItemsListButtonTemplate"
	scrollFrame.callback = self.FillListButton
    scrollFrame.list = displayList

    stash.tracking:SetCallback("item",function() self:Update() end)
end

function stash.itemsPanel:OnShow()
    self:UpdateNow()
    if settings.AutoFocus then
        stash.timer:Start(0,self.GrabSearchBoxFocus,self)
    end
    stash.main:RegisterEvent("MODIFIER_STATE_CHANGED")
end

function stash.itemsPanel:OnHide()
    stash.main:UnregisterEvent("MODIFIER_STATE_CHANGED")
end

-- main update procedure for this panel
function stash.itemsPanel:Update()
    stash.timer:Start(C.UI_UPDATE_DELAY,self.UpdateNow,self)
end

-- updates the UI immediately without a delay; except in rare cases, updates should
-- go through stash.itemsPanel:Update() to throttle calls to this function
function stash.itemsPanel:UpdateNow()
    if not self:IsVisible() then
        return -- panel isn't on screen, can ignore this request
    end
    self.Bottom:SetPoint("BOTTOMRIGHT",settings.LockWindow and 0 or -24, 0)
    if settings.IsFilterPanelOpen then
        stash.filterPanel:Show()
        stash.filterPanel:Resize()
        self.Top:SetHeight(30+2+stash.filterPanel:GetHeight())
    else
        self.Top:SetHeight(30)
        stash.filterPanel:Hide()
    end
    self.Top.FilterToggle:Update()
    self:PopulateList() -- refresh the list of items
    self.List:Update() -- update items list scrollframe content
    if stash.info:IsSearching() or settings.FilterRealm or settings.FilterType then
        self.Bottom.MoneyButton:Hide()
        self.Bottom.ResultsButton:Show()
        self.Bottom.ResultsButton.Text:SetText(format(L["%d \124cffffd200results for \124r%s"],#displayList,stash.filterPanel:GetActiveFilterList()))
    else
        self.Bottom.ResultsButton:Hide()
        self.Bottom.MoneyButton:Show()
        self.Bottom.MoneyButton.Money:SetAmount(stash.info:CountAllMoney()) -- update total money at bottom
    end
    if not stash.info:IsEverythingDataCached() then
        self:Update() -- if something not data cached, do a delayed update to come back
    end
end

-- callback for autoscrollframe to fill an item list button
function stash.itemsPanel:FillListButton(itemID)
    self.itemID = itemID
    local info = stash.info.items[itemID]
    self.Name:SetText(info[2])
    self.Count:SetText(BreakUpLargeNumbers(info[1] or 0))
    self.Icon:SetTexture(info[3])
    local color = C.QUALITY_COLORS[info[4] or 1] or C.QUALITY_COLORS[1]
    self.Name:SetTextColor(color.r,color.g,color.b)
    if settings.ShowRarityBorders then
        self.Rarity:Show()
        self.Rarity:SetVertexColor(color.r,color.g,color.b)
        self.Icon:SetTexCoord(0,1,0,1)
        self.Icon:SetSize(24,24)
        self.Icon:SetPoint("TOPLEFT")
    else
        self.Rarity:Hide()
        self.Icon:SetTexCoord(0.075,0.925,0.075,0.925)
        self.Icon:SetSize(23,23)
        self.Icon:SetPoint("TOPLEFT",1,-1)
    end
    self.Stripe:SetShown(self.index%2==0)
    if GetMouseFocus()==self then -- is mouse is over button when updated, pretend just entering it
        stash.itemsPanel.ListButtonOnEnter(self)
    end
end

-- updates the direction of the arrow on the FilterToggle button based on the IsFilterPanelOpen setting
function stash.itemsPanel.Top.FilterToggle:Update()
    local normalTexture = self:GetNormalTexture()
    local pushedTexture = self:GetPushedTexture()
    if settings.IsFilterPanelOpen then
        normalTexture:SetTexCoord(0.359375,0.546875,0.4296875,0.6171875)
        pushedTexture:SetTexCoord(0.5390625,0.7265625,0.4296875,0.6171875)
    else
        normalTexture:SetTexCoord(0,0.1875,0.4296875,0.6171875)
        pushedTexture:SetTexCoord(0.1796875,0.3671875,0.4296875,0.6171875)
    end
end

-- clicking the filter toggle in the topleft will open/close the filter pullout panel
function stash.itemsPanel.Top.FilterToggle:OnClick()
    settings.IsFilterPanelOpen = not settings.IsFilterPanelOpen
    stash.itemsPanel:UpdateNow()
end

-- clicking the settings toggle in topright will open the settings panel
function stash.itemsPanel.Top.SettingsToggle:OnClick()
    stash.itemsPanel:Hide()
    stash.settingsPanel:Show()
end

-- text changing in the search box will set the search mask and update list if mask changes
function stash.itemsPanel.Top.SearchBox:OnTextChanged(userInput)
    if stash.info:SetSearch(self:GetText()) then

        stash.itemsPanel:UpdateNow() -- doing an immediate update so it's more responsive (maybe option for delayed to call regular Update?)
    end
end

-- wipes searh (used by clear button and Reset button)
function stash.itemsPanel:ResetSearch()
    self.Top.SearchBox:SetText("")
    self.Top.SearchBox:ClearFocus()
    self.Top.SearchBox:GetScript("OnEditFocusLost")(self.Top.SearchBox)
end

-- returns true if the given itemID is an actual item or a battlepet
-- (not using IsEquippable because items that teach mounts and companion pets can be dressup'ed)
function stash.itemsPanel:IsDressUpable(itemID)
    if not itemID then
        return false
    elseif type(itemID)=="number" then
        return true
    else
        return itemID:match("^battlepet")
    end
end

-- mouse entering an item list button
function stash.itemsPanel:ListButtonOnEnter()
    self.Highlight:Show()
    self.Stripe:Hide()
    stash.tooltips:ShowItemTooltip(self,self.itemID)
    if IsModifiedClick("DRESSUP") and stash.itemsPanel:IsDressUpable(self.itemID) then
        ShowInspectCursor()
    else
        ResetCursor()
    end
end

-- mouse leaving an item list button
function stash.itemsPanel:ListButtonOnLeave()
    self.Highlight:Hide()
    self.Stripe:SetShown(self.index%2==0)
    stash.tooltips:HideTooltip()
    ResetCursor()
end

-- clicking an item will at most link to chat or show in dressing room
function stash.itemsPanel:ListButtonOnClick()
    local link
    if type(self.itemID)=="number" then -- if this is an item
        link = select(2,GetItemInfo(self.itemID))
    else -- if not an item, see if it's a battlepet and dressup modifier down
        if C.IS_MAINLINE then
            local speciesID = tonumber(self.itemID:match("battlepet:(%d+)"))
            if speciesID and IsModifiedClick("DRESSUP") then
                local _,_,_,creatureID,_,_,_,_,_,_,_,displayID = C_PetJournal.GetPetInfoBySpeciesID(speciesID)
                DressUpBattlePet(creatureID,displayID,speciesID)
            end
        end
        return -- only supporting links for items right now (fake pet links don't reflect stats)
    end
    if IsModifiedClick("CHATLINK") then
        ChatEdit_InsertLink(link)
    elseif IsModifiedClick("DRESSUP") then
        if C.IS_CLASSIC or C.IS_BCC then
            DressUpItemLink(link)
        else
            DressUpLink(link) -- this works for items that teaches pets and mounts also (retail only)
        end
    end
end

-- mouse entering the money total at the bottom of window
function stash.itemsPanel.Bottom.MoneyButton:OnEnter()
    if not settings.HideMoneyTooltip then
        stash.tooltips:ShowMoneyTooltip(self)
    end
end

-- mouse leaving the money total at the bottom of window
function stash.itemsPanel.Bottom.MoneyButton:OnLeave()
    stash.tooltips:HideTooltip()
end

-- click of money button opens moneyPanel
function stash.itemsPanel.Bottom.MoneyButton:OnClick()
    stash.itemsPanel:Hide()
    stash.moneyPanel:Show()
end

-- gathers a count of all items and potentially caches if first encountered
function stash.itemsPanel:PopulateList()
    stash.info:GatherAllItems()
    wipe(displayList)
    for itemID,itemInfo in pairs(stash.info.items) do
        if itemInfo[1]>0 then -- don't list items that have a 0 count
            tinsert(displayList,itemID)
        end
    end
    table.sort(displayList,settings.SortByRarity and self.SortByRarity or self.SortByName)
end

-- sets focus to the searchbox
function stash.itemsPanel:GrabSearchBoxFocus()
    self.Top.SearchBox:SetFocus(true)
end

-- sorts displayList by name
function stash.itemsPanel.SortByName(e1,e2)
    if e1 and not e2 then
        return true
    elseif not e1 and e2 then
        return false
    else
        return (stash.info.items[e1][C.INFO_NAME] or "") < (stash.info.items[e2][C.INFO_NAME] or "")
    end
end

-- sorts displayList by rarity first then name
function stash.itemsPanel.SortByRarity(e1,e2)
    if e1 and not e2 then
        return true
    elseif not e1 and e2 then
        return false
    else
        local r1 = stash.info.items[e1][C.INFO_RARITY] or 0
        local r2 = stash.info.items[e2][C.INFO_RARITY] or 0
        if r1 ~= r2 then
            return r1 > r2
        else
            return (stash.info.items[e1][C.INFO_NAME] or "") < (stash.info.items[e2][C.INFO_NAME] or "")
        end
    end
end
